

#ifndef _ICONDESK_HH_
#define _ICONDESK_HH_

#include <qwidget.h>
#include <qlist.h>
#include <qdir.h>
#include <qlined.h>

#include <sys/types.h>

#include "icon.hh"
#include "deskicon.hh"

class IconView ;

class IconDesk : public QWidget {

  Q_OBJECT ;

  friend IconView; 
  IconView *owner ;

  QList<Icon> *icon_list ;
  QListIterator<Icon> *icon_it ;
 
  QLineEdit *labelEdit ;
  QList<Icon> *selectedList ; // icons that are selected 
  
  QDir *current_dir ;
  time_t dir_last_mod ;

  void read (const QDir &) ;
  void read (const char *) ;

  QWidget *rootDesk ;

  Icon *focusIcon ; // icon currently in focus
  Icon *dragDropIcon ; // icon currently in drag drop

  int offsetx, offsety ; // scrolling offsets ;
  int vxsize, vysize ; // virtual size ;
  int scale ;
  
  QPoint pressPoint ; // position of last mouse press
  QPoint lastMousePoint ;
  QPoint pressOffset ;
  
  QList<DeskIcon> *dragWidgetList ;

  void add_new_icon (const DeskIcon &) ;
  void add_new_icon (const char *) ;
  void remove_icon (const Icon *ic) ;

protected:
  void paintEvent (QPaintEvent *) ;
  void mousePressEvent (QMouseEvent *) ;
  void mouseMoveEvent (QMouseEvent *) ;
  void mouseDoubleClickEvent (QMouseEvent *) ;
  void mouseReleaseEvent (QMouseEvent *) ;
  void resizeEvent (QResizeEvent *) ;

  void layoutIcons () ;
  void move_to_empty_space (Icon &ic) ;
  void initDrag (Icon *ic) ;
  Icon *point_is_in_icon (const QPoint &p) const ;
  Icon *point_is_in_icon_label (const QPoint &p) const ;
  bool point_is_in_icon (const QPoint &p, const Icon* ic) const ;
  bool rect_meets_icon (const QRect &r) ;
  void drawIcons (const QRect &r, bool) ;
  void single_repaint (const Icon *) ;
  void unselectAll () ;
  void select (Icon *ic) ;
  
  int appropSize() const ;
  
  void save_locations () ;

  QTimer *refresh_timer ;

  signals:
  void resized (int, int) ;
  void dragPointAt (const QPoint &, const QList<DeskIcon> &) ;
  void dragDropRelease (const QPoint &, const QList<DeskIcon> &) ;
  void newDirWindow (const char *) ;
  void broadcast_file_change (const QFileInfo &, const QFileInfo &) ;
  void signal_refresh () ;

public:
  IconDesk (IconView *parent) ;
  const QList<Icon> *getSelectedList () const { return selectedList ; }
  void checkDragDrop (QPoint &p) ;
  void drag_drop_happened (const QPoint &, const QList<DeskIcon> &) ;
  void unHighlightDragDrop () ;

 public slots:
 void glueLabel (Icon *) ;
  void rescale (int s) ;
  void horizScroll (int) ;
  void verticScroll (int) ;
  void rename_by_label () ;
  void slot_file_change (const QFileInfo &, const QFileInfo &) ;
  void slot_refresh () ;

} ;


#endif // _ICONDESK_HH_

